/**
 * Copyright (c) 2006, 2009 Bob Swift
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *              notice, this list of conditions and the following disclaimer in the
 *            documentation and/or other materials provided with the distribution.
 *     * The names of contributors may not be used to endorse or promote products
 *           derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * Created on Jan 19, 2006 by Bob Swift
 */

package org.swift.jira.cli;

import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import org.swift.common.cli.AbstractRestClient;
import org.swift.common.cli.CliUtils;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import com.martiansoftware.jsap.JSAPResult;
import com.martiansoftware.jsap.SimpleJSAP;

/**
 * Rest Client - this is a implementation to use as a guest for the jira soap client
 */
public class JiraRestClient extends AbstractRestClient {

    protected Document doc = null; // parsed result stream.
    protected String jiraResponse; // hack necessary for JIRA

    /**
     * Guest constructor
     */
    public JiraRestClient(final SimpleJSAP jsap, final JSAPResult jsapResult) {
        this.jsap = jsap;
        this.jsapResult = jsapResult;
        setVerbose(jsapResult.getBoolean("verbose"));
        setDebug(jsapResult.getBoolean("debug"));
        // setupLogging(); // setup log4j properties
        // this is what JIRA needs for running requests
        token = "os_username=" + getString("user") + "&os_password=" + getString("password");
    }

    @Override
    protected String getDefaultServiceExtension() {
        return "";
    }

    /**
     * Get the key used to retrieve the login token
     * 
     * @result key used to retrieve the login token from the response message
     */
    @Override
    protected String getLoginTokenKey() {
        return null;
    }

    /**
     * Get key use in response for an error conditions
     * 
     * @result key used to retrieve error from response
     */
    @Override
    protected String getErrorKey() {
        return "errMsg";
    }

    /**
     * Get the service - default to return the service parameter. Override this for guest clients!
     */
    @Override
    protected String getService() {
        return "";
    }

    /**
     * Get any qualifier (postfix) that is added to all command requests to the server
     * 
     * @result valued added to the command before building the URL
     */
    @Override
    protected String getActionQualifier() {
        return "";
    }

    /**
     * Until JIRA has some real rest interfaces, we are hacking it - TODO: use real REST interfaces when available. For now, just use the raw data and regex
     * analysis. These interfaces will likely change over time.
     * 
     * @param response - data returned after a remote request
     * @return void - jiraResponse is updated
     */
    @Override
    protected void parseResultData(final String response) throws ParserConfigurationException, IOException, SAXException, RemoteRestException {

        // out.println("response: " + response);
        String[] knownErrors = {">Session Expired<", "Cannot find link matching these parameters.",
                "You do not have permission to link issues in this project."};

        for (String string : knownErrors) {
            if (response.contains(string)) {
                throw new RemoteRestException("JIRA reports: " + string.replaceAll("[<>]", ""));
            }
        }

        String[] knownErrors2 = {" is required.", "nvalid link type."};

        for (String string : knownErrors2) {
            String result = CliUtils.matchRegex(response, "<div [^>]*class=\"error\">([^<]*" + string + "[^<]*)</div>");
            if (!"".equals(result)) {
                // out.println("result: " + result);
                throw new RemoteRestException(result);
            }
        }

        // <div class="error">Description is required.</div> for required parameters on create issue
        // String result = CliUtils.matchRegex(response, "<div class=\"error\">([^<]* is required.)</div>");
        // String result = CliUtils.matchRegex(response, "<div [^>]*class=\"error\">([^<]* )</div>");
        // if (!"".equals(result)) {
        // out.println("result: " + result);
        // throw new RemoteRestException(result);
        // }

        // JIRA forces us to a login screen, probably a permission problem
        if (response.contains("<form method=\"POST\" action=\"/login.jsp\" name=\"loginform\">")) {
            throw new RemoteRestException("Possible permission issue. Verify user has permission for the action.");
        }

        jiraResponse = response;
        return;
    }

    /**
     * Get value text identified by key from the response already available
     * 
     * <pre>
     * Example - component
     * &lt;span class=&quot;errMsg&quot;&gt;A component with the name C1c already exists in this project.&lt;
     * Example - create subtask error
     * &lt;span class=&quot;errMsg&quot;&gt;Reporter is required.&lt;/span&gt;
     * Example - create subtask success - get key
     * /browse/JIRACLI-17
     * </pre>
     * 
     * @param key - to get value for from the response
     * @return value of the requested key from the response, blank is returned if the key is not found
     */
    @Override
    public String getValue(final String key) throws ClientException {
        return getValue(key, null);
    }

    /**
     * Get value text identified by key from the response already available
     * 
     * @param key - to get value for from the response
     * @param searchData - additional info to use in the regex for finding the value requested
     * @return value of the requested key from the response, blank is returned if the key is not found
     */
    public String getValue(final String key, final String searchData) throws ClientException {

        String result = "";
        if (key.equals(getErrorKey())) {
            result = CliUtils.matchRegex(jiraResponse, "<span class=\"" + key + "\">([^<]*)</span>.*"); // stop on any character not <)
            if (result.equals("")) {
                result = CliUtils.matchRegex(jiraResponse, "<div class=\"errorArea\">.*"); // form errors
            }
        } else if (key.equals("issueKey")) {
            result = CliUtils.matchRegex(jiraResponse, "/browse/([a-zA-Z][a-zA-Z0-9]+-[0-9]+)?.*"); // issue key
        } else if (key.equals("linkId")) {
            /**
             * <pre>
             * <tr> ...
             *      <td valign=top><b>duplicate</b></td>
             *      <td valign=top>duplicates</td>
             *      <td valign=top>is duplicated by</td>
             *      </td>
             *      <td valign=top width=1% nowrap>
             *          <a href="EditLinkType!default.jspa?id=10010">Edit</a>
             *          |
             *          <a id="del_duplicate" href="DeleteLinkType!default.jspa?id=10010">Delete</a>
             *      </td>
             * </tr>
             * </pre>
             */
            String regexString = "<td .*>" + searchData + "</td>.*EditLinkType[^=]*id=(\\d+).*>";
            result = CliUtils.matchRegex(jiraResponse, regexString); // link id
            if (verbose) {
                // out.println("regex: " + regexString);
                out.println("Link id: " + result);
            }
        }
        return result;
    }
}