/*
 * Copyright (c) 2009 Bob Swift.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *              notice, this list of conditions and the following disclaimer in the
 *            documentation and/or other materials provided with the distribution.
 *     * The names of contributors may not be used to endorse or promote products
 *           derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  Created on: Sep 16, 2009
 *      Author: Bob Swift
 */

package org.swift.jira.cli;

import java.io.PrintStream;
import java.rmi.RemoteException;

import org.apache.axis.AxisFault;
import org.swift.common.cli.AbstractUserHelper;
import org.swift.common.cli.CliClient.ClientException;

import com.dolby.jira.net.soap.jira.JiraSoapService;
import com.dolby.jira.net.soap.jira.RemoteGroup;
import com.dolby.jira.net.soap.jira.RemotePermissionException;
import com.dolby.jira.net.soap.jira.RemoteUser;
import com.dolby.jira.net.soap.jira.RemoteValidationException;

public class JiraUserHelper extends AbstractUserHelper {

    protected PrintStream out = System.out;
    protected JiraSoapService service;
    protected String token;

    public JiraUserHelper(JiraClient client, boolean verbose, JiraSoapService service, String token) {
        super(client, verbose);
        this.service = service;
        this.token = token; // authorization token
    }

    /**
     * Checks if userid is already defined
     * 
     * @param userid
     * @return true if defined, false otherwise
     */
    @Override
    public boolean hasUser(final String userId) throws RemoteException {
        try {
            return service.getUser(token, userId) != null;
        } catch (AxisFault exception) { // Guard against occasional NPE from service when userId is not defined
            if (!exception.getFaultString().contains("NullPointerException")) {
                throw exception;
            }
        }
        return false;
    }

    /**
     * Checks if group is already defined
     * 
     * @param group
     * @return true if defined, false otherwise
     */
    @Override
    public boolean hasGroup(final String group) throws RemoteException {
        try {
            return (null != service.getGroup(token, group));
        } catch (RemoteValidationException ignore) {
        }
        return false;
    }

    /**
     * Adds a user after most validation has occurred. Subclass may have additional validation and must implement remote service
     * 
     * @param userId
     * @param userPassword
     * @param userEmail
     * @param userFullName
     */
    @Override
    public void addUserService(final String userId, final String userPassword, final String userEmail, final String userFullName) throws RemoteException,
            ClientException {
        if (userEmail == "") { // JIRA has this additional requirement
            throw new ClientException("A non-blank userEmail is required."); // cover for from file case
        }
        service.createUser(token, userId, userPassword, userFullName, userEmail);
    }

    /**
     * Removes a user. Info message if not found. after most validation has occurred. Subclass may have additional validation and must implement remote service
     * 
     * @param userId to remove
     */
    @Override
    public void removeUserService(final String userId) throws RemoteException, ClientException {
        try {
            service.deleteUser(token, userId);
        } catch (RemotePermissionException exception) {
            throw new RemoteException("Remove user failed. Users who are assignee or reporter for an issue cannot be removed.");
            // throw new RemoteException(exception.toString()); // ugly message
        }
    }

    /**
     * Add a group after most validation has occurred. Subclass may have additional validation and must implement remote service
     * 
     * @param group to add
     */
    @Override
    public void addGroupService(final String group) throws RemoteException, ClientException {
        service.createGroup(token, group, null);
    }

    /**
     * Removes a group. Info message if not found. after most validation has occurred. Subclass may have additional validation and must implement remote service
     * 
     * @param group to remove
     * @param defaultGroup - users in group will be moved to the defaultGroup if specified
     */
    @Override
    public void removeGroupService(final String group, final String defaultGroup) throws RemoteException, ClientException {
        service.deleteGroup(token, group, defaultGroup);
    }

    /**
     * Add user to group after most validation has occurred. Subclass may have additional validation and must implement remote service. Exception is thrown if
     * userId or group are invalid
     * 
     * @param userId
     * @param group
     */
    @Override
    public void addUserToGroupService(final String userId, final String group) throws RemoteException, ClientException {
        RemoteUser remoteUser = service.getUser(token, userId);
        if (remoteUser == null) {
            throw new RemoteException("User: " + userId + " is not defined.");
        } else {
            RemoteGroup remoteGroup = service.getGroup(token, group);
            if (remoteGroup == null) {
                throw new RemoteException("Group: " + group + " is not defined.");
            } else {
                service.addUserToGroup(token, remoteGroup, remoteUser);
            }
        }
    }

    /**
     * Removes user from group. Info message if not found. after most validation has occurred. Subclass may have additional validation and must implement remote
     * service
     * 
     * @param userId
     * @param group
     */
    @Override
    public boolean removeUserFromGroupService(final String userId, final String group) throws RemoteException, ClientException {
        assert userId != null;
        assert group != null;
        boolean result = false;
        RemoteGroup remoteGroup = service.getGroup(token, group);
        if (remoteGroup == null) {
            // out.println("Group: " + group + " is not defined.");
            throw new RemoteException("Group: " + group + " is not defined.");
        }
        RemoteUser remoteUser = null;
        try {
            remoteUser = service.getUser(token, userId);
        } catch (AxisFault exception) { // Guard against occasional npe from service
            if (!exception.getFaultString().contains("NullPointerException")) {
                throw exception;
            }
        }
        if (remoteUser == null) {
            out.println("User: " + userId + " is not defined.");
            // throw new RemoteException("User: " + userId + " is not defined.");
        } else {
            // RemoteGroup remoteGroup = service.getGroup(token, group);
            // if (remoteGroup == null) {
            // out.println("Group: " + group + " is not defined.");
            // throw new RemoteException("Group: " + group + " is not defined.");
            // } else {
            service.removeUserFromGroup(token, remoteGroup, remoteUser);
            result = true;
            // }
        }
        return result;
    }
}
